package cn.iocoder.yudao.module.member.service.memberUserScore;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApi;
import cn.iocoder.yudao.module.member.api.score.MemberUserScoreApiImpl;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateReqDTO;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreOperateRespDTO;
import cn.iocoder.yudao.module.member.api.score.dto.MemberUserScoreUpdateReqDTO;
import cn.iocoder.yudao.module.member.dal.dataobject.memberUserScore.MemberUserScoreDO;
import cn.iocoder.yudao.module.member.dal.mysql.memberUserScore.MemberUserScoreMapper;
import cn.iocoder.yudao.module.member.enums.ScoreOperateTypeEnum;
import cn.iocoder.yudao.module.member.enums.ScoreSourceTypeEnum;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreBackVO;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreOperateQueryVO;
import cn.iocoder.yudao.module.member.vo.memberUserScore.MemberUserScoreQueryVO;
import com.alibaba.excel.util.CollectionUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.member.enums.ErrorCodeConstants.*;

/**
 * 会员积分 Service 实现类
 *
 * @author 系统管理员
 */
@Service
@Validated
public class MemberUserScoreServiceImpl extends AbstractService<MemberUserScoreMapper, MemberUserScoreDO> implements MemberUserScoreService {

    @Resource
    private MemberUserScoreMapper userScoreMapper;

    @Resource
    private MemberUserScoreApiImpl scoreApi;

    @Override
    public void deleteUserScore(Long id) {
        // 删除
        userScoreMapper.deleteById(id);
    }

    @Override
    public MemberUserScoreDO getUserScore(Long id) {
        return userScoreMapper.selectById(id);
    }

    @Override
    public List<MemberUserScoreDO> getUserScoreList(Collection<Long> ids) {
        return userScoreMapper.selectBatchIds(ids);
    }

    @Override
    public PageResult<MemberUserScoreBackVO> getUserScorePage(MemberUserScoreQueryVO query) {
        int start = (query.getPageNo() - 1) * query.getPageSize();
        int size = query.getPageSize();
        List<MemberUserScoreBackVO> list = userScoreMapper.getUserScoreList(start, size, query);
        int total = userScoreMapper.countUserScore(query);
        return new PageResult<>(list, total, query.getPageSize(), query.getPageNo(), (total + query.getPageSize() - 1) / query.getPageSize());
    }

    @Override
    public List<MemberUserScoreDO> getUserScoreList(MemberUserScoreQueryVO query) {
        return userScoreMapper.selectList(query);
    }
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void operate(MemberUserScoreOperateQueryVO query) {
        if (CollectionUtils.isEmpty(query.getMemberIds())) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        if (query.getScoreCount() <= 0) {
            throw exception(SCORE_COUNT_ERROR);
        }
        // 校验用户当前积分是否满足扣减要求
        if (query.getOperateType() == 2) {
            LambdaQueryWrapper<MemberUserScoreDO> wrapper = Wrappers.lambdaQuery();
            wrapper.in(MemberUserScoreDO::getMemberId, query.getMemberIds());
            List<MemberUserScoreDO> userScoreDOList = this.list(wrapper);
            List<MemberUserScoreDO> notEnoughScoreList = userScoreDOList.stream()
                    .filter(item -> item.getHoldScore() < query.getScoreCount()).collect(Collectors.toList());
            if (!notEnoughScoreList.isEmpty()) {
                throw exception(MEMBER_SCORE_NOT_ENOUGH);
            }
        }
        query.getMemberIds().forEach(memberId -> {
            Map<String,Object> extParam = new HashMap<>();
            extParam.put("comment", query.getComment());
            MemberUserScoreOperateRespDTO memberUserScoreOperateRespDTO = scoreApi.operateScore(MemberUserScoreOperateReqDTO.builder()
                    .memberId(memberId)
                    .scoreCount(query.getScoreCount())
                    .operateType(ScoreOperateTypeEnum.parseByValue(query.getOperateType()))
                    .sourceType(ScoreSourceTypeEnum.MANUAL_OPERATE)
                    .extParam(extParam)
                    .build());
        });
    }

    @Override
    public void updateUserScore(MemberUserScoreUpdateReqDTO query) {
        // 增加积分，只需要增加持有积分即可
        // 减少积分，持有积分减少， 已兑换或者已失效积分增加
        if (query.getMemberId() == null) {
            throw exception(MEMBER_ID_IS_NULL);
        }
        MemberUserScoreDO memberUserScoreDO = userScoreMapper.getByMemberId(query.getMemberId());
        if (memberUserScoreDO == null) {
            memberUserScoreDO = initMemberUserScore(query.getMemberId());
        }
        if (query.getOperateType() == ScoreOperateTypeEnum.ADD) {
            memberUserScoreDO.setHoldScore(memberUserScoreDO.getHoldScore() + query.getScoreCount());
            this.saveOrUpdate(memberUserScoreDO);
        }
        if (query.getOperateType() == ScoreOperateTypeEnum.REDUCE) {
            if (memberUserScoreDO.getHoldScore() < query.getScoreCount()) {
                throw exception(MEMBER_SCORE_NOT_ENOUGH);
            }
            memberUserScoreDO.setHoldScore(memberUserScoreDO.getHoldScore() - query.getScoreCount());
            if (ScoreSourceTypeEnum.getInvalidScoreStatusSourceType().contains(query.getSourceType())) {
                memberUserScoreDO.setExpiredScore(memberUserScoreDO.getExpiredScore() + query.getScoreCount());
            } else {
                memberUserScoreDO.setUsedScore(memberUserScoreDO.getUsedScore() + query.getScoreCount());
            }
            this.saveOrUpdate(memberUserScoreDO);
        }
    }

    private MemberUserScoreDO initMemberUserScore(Long memberId) {
        MemberUserScoreDO memberUserScoreDO = new MemberUserScoreDO();
        memberUserScoreDO.setMemberId(memberId);
        memberUserScoreDO.setHoldScore(0);
        memberUserScoreDO.setExpiredScore(0);
        memberUserScoreDO.setUsedScore(0);
        memberUserScoreDO.setCreateTime(new Date());
        return memberUserScoreDO;
    }
}
